# Copyright (c) 2007 The Regents of the University of California.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#   1. Redistributions of source code must retain the above copyright
#      notice, this list of conditions, and the following disclaimer.
#   2. Redistributions in binary form must reproduce the above
#      copyright notice, this list of conditions, and the following
#      disclaimer in the documentation and/or other materials provided
#      with the distribution.
#   3. Redistributions must acknowledge that this software was
#      originally developed by the UCSF Computer Graphics Laboratory
#      under support by the NIH National Center for Research Resources,
#      grant P41-RR01081.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER "AS IS" AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
# OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import string
from Loop import *
from Segment import *
from Tools import printVerbose

class Input:
	def __init__(self,input,verbose):
		self.name = input
		self.image = []
		self.bridge = []
		self.feature = []
		self.zero = 0
		self.farend = 0.0

		# get the segment information from input			
		f = open(self.name)
		lcount = 0
		self.segs = []
		self.loops = []
		self.image = []
		self.outsidedo = ''
		self.cysdo = ''
		self.cyssym = 0
		self.cyscol = ''
		self.glydo = ''
		self.glysym = 0
		self.glycol = ''
		self.memdo = ''
		self.chdo = ''
		self.acidsym = 0
		self.acidcol = ''
		self.basesym = 0
		self.basecol = ''
		self.amdo = ''
		self.amsym = 0
		self.amcol = ''
		self.ohdo = ''
		self.ohsym = 0
		self.ohcol = ''
		self.aromdo = ''
		self.aromsym = 0
		self.aromcol = ''
		self.prodo = ''
		self.prosym = 0
		self.procol = ''
		self.sigdo = ''
                self.signum = 0
		self.sigstart = []
		self.sigend = []
		self.sigsym = []
		self.sigcol = []
		self.spdo = '' 
		self.spnum = 0
		self.sppos = []
		self.spsym = []
		self.spcol = []
		self.cusdo = ''
		self.cusnum = 0
		self.cusloop = []
		self.cuselements = []
		self.cusstart = []
		self.cusend = []
		while 1:
			line = f.readline()
			if not line:
				break
			if lcount == 0:
				# extract relavent information from first line 
				# seq file name, number of segments, start side
				fields = string.split(line)
				if len(fields) != 4:
					print 'error in the input file'
				self.file = fields[0]
				self.numsegs = int(fields[1])
				self.side = int(fields[2])
				self.protein_name = fields[3]
				msg="%s contains %d segments" %(self.protein_name,self.numsegs)
			        printVerbose(verbose,msg)

				# get the sequence information from file
				f2 = open(self.file,'r')
				count = 0
				s = ''
				while 1:
					line = f2.readline()
                			if not line:
                        			break
					if count> 0:
	                        		x = len(line)
        	                		#print "x is %d" % x
 	               	        		y = line[0:x-1]
                        			s = s + y
                			count = count + 1
				f2.close()
				self.seq = s

			if lcount > 0:
				#get the segment information
				if lcount <= self.numsegs:
					# extract relavent segment information from 
					# additional lines of input
					newseg = Segment(lcount,line)
					self.segs.append(newseg)
                                        if lcount == self.numsegs:
						self._get_order(verbose)
						self._get_data(verbose)

				else:
					# read in display options
					# show external data (cysteine data and glycosylation data)
					if lcount == self.numsegs +1:
						outfields = string.split(line)
						self.outsidedo = outfields[0]
						if self.outsidedo == 'y':
							ocount = 1
							self.cysdo = outfields[ocount]
							if self.cysdo == 'y':
								self.cyssym = int(outfields[ocount+1])
								self.cyscol = outfields[ocount+2]
								ocount = ocount + 3
								#print "cyssym is %d and color is %s" % (self.cyssym,self.cyscol)
							else:
								ocount = ocount + 1

							self.glydo = outfields[ocount]
							if self.glydo == 'y':
								self.glysym = int(outfields[ocount+1])
								self.glycol = outfields[ocount+2]
								#print "glysym is %d and color is %s" % (self.glysym,self.glycol)
					# show membrane data
					if lcount == self.numsegs +2:
						memfields = string.split(line)
						self.memdo = memfields[0]
						if self.memdo == 'y':
							count = 1
							self.chdo = memfields[count]
							if self.chdo == 'y':							
								self.acidsym = int(memfields[count+1])
								self.acidcol = memfields[count+2]
								self.basesym = int(memfields[count+3])
								self.basecol = memfields[count+4]
								count = count + 5
							else:
								count= count + 1
							self.amdo = memfields[count]
							if self.amdo == 'y':
								self.amsym = int(memfields[count+1])
								self.amcol = memfields[count+2]
								count = count + 3
							else:
								count = count + 1
							self.ohdo = memfields[count]
							if self.ohdo == 'y':
								self.ohsym = int(memfields[count+1])
								self.ohcol = memfields[count+2]
								count = count + 3
							else:
								count = count + 1
							self.aromdo = memfields[count]
							if self.aromdo == 'y':
								self.aromsym = int(memfields[count+1])
								self.aromcol = memfields[count+2]
								count = count + 3
							else:
								count = count + 1
							self.prodo = memfields[count]
							if self.prodo == 'y':
								self.prosym = int(memfields[count+1])
								self.procol = memfields[count+2]

					# show signature data
					if lcount == self.numsegs +3:
						sigfields = string.split(line)
						self.sigdo = sigfields[0]
						if self.sigdo == 'y':
							self.signum = int(sigfields[1])						
							sigcts = 0
							while sigcts < self.signum:
								sigs = int(sigfields[4*sigcts+2])
								self.sigstart.append(sigs)
								sige = int(sigfields[4*sigcts+3])
								self.sigend.append(sige)
								sigsm = int(sigfields[4*sigcts+4])
								self.sigsym.append(sigsm)
								sigc = sigfields[4*sigcts+5]
								self.sigcol.append(sigc)
								sigcts = sigcts + 1	
					# show special data
					if lcount == self.numsegs +4:
						spfields = string.split(line)
						self.spdo = spfields[0]
						if self.spdo == 'y':
							self.spnum = int(spfields[1])						
							spcts = 0
							while spcts < self.spnum:
								spp = int(spfields[3*spcts+2])
								self.sppos.append(spp)
								sps = int(spfields[3*spcts+3])
								self.spsym.append(sps)
								spc = spfields[3*spcts+4]
								self.spcol.append(spc)
								spcts = spcts + 1	

					# show custom loop data
					if lcount == self.numsegs + 5:
						cusfields = string.split(line)
						self.cusdo = cusfields[0]
						if self.cusdo == "y":
							self.cusnum = int(cusfields[1])
							#print "there are %d custom loop(s)"  %self.cusnum
							cuscts = 0
							cus_segments = 0
							while cuscts < self.cusnum:
								cusloopnum = int(cusfields[cus_segments+2])
								#print "custom loop %d is loop %d of the image" %(cuscts,cusloopnum)
								self.cusloop.append(cusloopnum)
								elements = int(cusfields[cus_segments+3])
								#print "loop %d contains %d elements"  %(cuscts,elements)
								self.cuselements.append(elements)
								cycle = 0
								while cycle < elements:
									cuss = int(cusfields[cus_segments+cycle*2+4])
									#print "for custom loop %d element %d starts at %d"  %(cuscts,cycle,cuss)
									self.cusstart.append(cuss)
									cuse = int(cusfields[cus_segments+cycle*2+5])
									#print "for custom loop %d element %d ends at %d"  %(cuscts,cycle,cuse)
									self.cusend.append(cuse)
									cycle = cycle + 1
								cus_segments = cus_segments +elements*2+2
								cuscts = cuscts + 1
									
			lcount = lcount + 1
		f.close()

		return

	def _get_data(self,verbose):
		# determine the beginning section
 		j = 0
		k = 0
		if self.numsegs <= 0:
			msg = "problem with initial data"
			printVerbose(verbose,mag)
		else:
			# have data to work with
			start = 1
			end = self.segs[0].startSeg - 1
			if self.side == 1:
				msg = " protein starts on the outside"
			else:
				msg = " protein starts on the inside"
			printVerbose(verbose,msg)

			# fill in information on the starting loop
			if start == 1 and end == 0:
				start = 0
			
			loop = Loop(k,start,end,0,self.side)
			self.loops.append(loop)
			#print "beginning loop %d starts at %d and ends at %d" % (k,self.loops[k].startLoop,self.loops[k].endLoop)
			k = k + 1

			# work with the chains  
			#print "-|there are %d transmembrane segments|-" % self.numsegs

			while j < self.numsegs:

				if self.segs[j].typeSeg == 1:
					# completely crosses the membrane
					msg = "   segment %d crosses the membrane" % j
					printVerbose(verbose,msg)					
				else:
					# partial crossing of membrane
					msg = "   segment %d partially crosses the membrane" % j					
					printVerbose(verbose,msg)					
										
				if j == self.numsegs -1:
				# stop if number of segments reached
					break
			
				# work with connecting loops
				conupstart = self.segs[j].endSeg + 1
				conupend = self.segs[j+1].startSeg - 1
				# fill in information about the connecting loop
				whichside = self.segs[k-1].dir
				if whichside == 1:
					if self.segs[j].typeSeg == 2:
						side = 1
						where = 'outside'
					else:
						side = 2
						where = 'inside'
				else:
					if self.segs[j].typeSeg == 2:
						side = 2
						where = 'inside'
					else:
						side = 1
						where = 'outside'

				loop = Loop(k,conupstart,conupend,1,side)
	                	self.loops.append(loop)
				#print "loop %d starts at %d and ends at %d" %  (k,self.loops[k].startLoop,self.loops[k].endLoop)
				#print "loop %d is on the %s" % (j+1,where)
				k = k + 1
				j = j + 1

			# finish up the image
			backstart = self.segs[self.numsegs-1].endSeg + 1
			backend = len(self.seq)
			#print "the last region starts at %d and ends at %d" % (backstart,backend)
			whichside = self.segs[k-1].dir
			if whichside == 1:
				side = 2
				where = 'inside'
			else:
				side = 1
				where = 'outside'
			loop = Loop(k,backstart,backend,0,side)
                	self.loops.append(loop)
			#print "ending loop %d starts at %d and ends at %d" % (k,self.loops[k].startLoop,self.loops[k].endLoop)
			#print "the end is on the %s" % where
		return


	def _get_order(self,verbose):
		# determine the direction of the segments
		i = 1
		while i <= self.numsegs:
			if i == 1 and self.side == 1:
				self.segs[0].dir = 1	
			elif i == 1 and self.side == 2:
				self.segs[0].dir = 2
			elif i > 1:
				if self.segs[i-2].typeSeg == 1 and self.segs[i-1].typeSeg == 1:
					if self.segs[i-2].dir == 1:
						self.segs[i-1].dir = 2
					else:
						self.segs[i-1].dir = 1
				elif self.segs[i-2].typeSeg == 1 and self.segs[i-1].typeSeg == 2: 
					if self.segs[i-2].dir == 1:
						self.segs[i-1].dir = 2
					else:
						self.segs[i-1].dir = 1
				elif self.segs[i-2].typeSeg == 2 and self.segs[i-1].typeSeg == 1:
					if self.segs[i-2].dir == 1:
						self.segs[i-1].dir = 1
					else:
						self.segs[i-1].dir = 2
				elif self.segs[i-2].typeSeg == 2 and self.segs[i-1].typeSeg == 2:
					if self.segs[i-2].dir == 1:
						self.segs[i-1].dir = 1
					else:
						self.segs[i-1].dir = 2

			msg = "   direction of segment %d is %d" % (i,self.segs[i-1].dir)
			printVerbose(verbose,msg)
			i = i + 1

		return

