# Copyright (c) 2007 The Regents of the University of California.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#   1. Redistributions of source code must retain the above copyright
#      notice, this list of conditions, and the following disclaimer.
#   2. Redistributions in binary form must reproduce the above
#      copyright notice, this list of conditions, and the following
#      disclaimer in the documentation and/or other materials provided
#      with the distribution.
#   3. Redistributions must acknowledge that this software was
#      originally developed by the UCSF Computer Graphics Laboratory
#      under support by the NIH National Center for Research Resources,
#      grant P41-RR01081.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER "AS IS" AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
# OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from Tools import * 

class Bridge:
	"a class for zerobridge information"
	def __init__(self,line):
                self.line = ''
                self.line = line
                return

class Feature:
	"a class for feature information"
	def __init__(self,line):
                self.line = ''
                self.line = line
                return

def makefeaturebox(r,whichside,x,y,patch,num,verbose):
	#parameters
	#R - symbol radius
	if whichside == 2:
		sign = 1.0
	if whichside == 1:
		sign = -1.0

	if num == 2:
		spacer = 2.15
	if num == 3:
		spacer = 3.25
	if num == 4:
		spacer = 4.35
	R = 5.0
	f_start_x = x - 1.5*R
	f_start_y = y - 2.0*float(patch)*R*sign - R*sign
	f_ltop_x = f_start_x
	f_ltop_y = y - R*sign
	f_rtop_x = f_start_x + num*2.0*R + spacer*R 
	f_rtop_y = f_ltop_y
	f_end_x = f_rtop_x
	f_end_y = f_start_y

	line = "%f %f %f %f %f %f %f %f" %(f_start_x,f_start_y,f_ltop_x,f_ltop_y,f_rtop_x,f_rtop_y,f_end_x,f_end_y)
	msg = line
	printVerbose(verbose,msg)
	linex = Feature(line)
	r.feature.append(linex)

	return
 
def makebridge(r,x,whichside,verbose):
	#parameters 
	#W - width between the centers of transmembrane segments
	#R - symbol radius
        #B - buffer width of symbol
        #sl -  size of layer is 3.5

	R = 5.0
	W = 16.0*R
	B = 1
	sl = 3.5

	if whichside == 1:
                ybase = 0.0 + R
                dir = 1
        else:
                ybase = 0.0 - (2.0*(B + .5*sl) + 5.0*(sl + B) + 1)*R
                dir = -1


	b_start_x = x
	b_start_y = ybase 
	b_ltop_x = x
	b_ltop_y = ybase + dir*2*R
	b_rtop_x = x + W
	b_rtop_y = ybase + dir*2*R
	b_end_x = x + W
	b_end_y = ybase

	line = "%f %f %f %f %f %f %f %f" %(b_start_x,b_start_y,b_ltop_x,b_ltop_y,b_rtop_x,b_rtop_y,b_end_x,b_end_y)
	msg = line
	printVerbose(verbose,msg)
	linex = Bridge(line)
	r.bridge.append(linex)
	r.zero = b_ltop_y

	return

