# Copyright (c) 2007 The Regents of the University of California.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#   1. Redistributions of source code must retain the above copyright
#      notice, this list of conditions, and the following disclaimer.
#   2. Redistributions in binary form must reproduce the above
#      copyright notice, this list of conditions, and the following
#      disclaimer in the documentation and/or other materials provided
#      with the distribution.
#   3. Redistributions must acknowledge that this software was
#      originally developed by the UCSF Computer Graphics Laboratory
#      under support by the NIH National Center for Research Resources,
#      grant P41-RR01081.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER "AS IS" AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
# OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from xmllib import XMLParser

class Topo:
	pass

class Location:
	pass

class TopoParser(XMLParser):

	def __init__(self, *args, **kw):
		apply(XMLParser.__init__, (self,) + args, kw)
		self.attributes = {
			"TOPO":	{
				"name":		None,
			},
			"bbox":	{
				"minx":		None,
				"maxx":		None,
				"miny":		None,
				"maxy":		None,
			},
			"layout": {
				"radius":	"5.0",
				"label_top":	"",
				"label_bot":	"",
			},
			"membrane":	{
				"m_minx":	None,
				"m_maxx":	None,
				"m_topy":	None,
				"m_boty":	None,
			},
			"zerobridge":	{
				"b_start_x":	None,
				"b_start_y":	None,
				"b_ltop_x":	None,
				"b_ltop_y":	None,
				"b_rtop_x":	None,
				"b_rtop_y":	None,
				"b_end_x":	None,
				"b_end_y":	None,
			},
			"feature":	{
				"f_start_x":	None,
				"f_start_y":	None,
				"f_ltop_x":	None,
				"f_ltop_y":	None,
				"f_rtop_x":	None,
				"f_rtop_y":	None,
				"f_end_x":	None,
				"f_end_y":	None,
			},
			"entry":	{
				"position":	None,
				"residue":	None,
				"x":		None,
				"y":		None,
				"symbol":	"1",
				"foreground":	"0 0 0",
				"background":	"1 1 1",
				"border":	"0 0 0",
			},
		}
		self.elements = {
			"TOPO": (self.topo_start, self.topo_end),
			"bbox": (self.bbox_start, self.bbox_end),
			"layout": (self.layout_start, self.layout_end),
			"membrane": (self.membrane_start, self.membrane_end),
			"zerobridge": (self.zerobridge_start, self.zerobridge_end),
			"feature": (self.feature_start, self.feature_end),
			"entry": (self.entry_start, self.entry_end),
		}

	def topo_start(self, attr):
		self.topo = Topo()
		self.topo.name = attr["name"]
		self.topo.locations = []
	def topo_end(self):
		pass

	def bbox_start(self, attr):
		self.topo.xMin = float(attr["minx"])
		self.topo.xMax = float(attr["maxx"])
		self.topo.yMin = float(attr["miny"])
		self.topo.yMax = float(attr["maxy"])
	def bbox_end(self):
		pass

	def layout_start(self, attr):
		self.topo.radius = float(attr["radius"])
		self.topo.labelTop = attr["label_top"]
		self.topo.labelBottom = attr["label_bot"]
	def layout_end(self):
		pass

	def membrane_start(self, attr):
		self.topo.xMembMin = float(attr["m_minx"])
		self.topo.xMembMax = float(attr["m_maxx"])
		self.topo.yMembTop = float(attr["m_topy"])
		self.topo.yMembBottom = float(attr["m_boty"])
	def membrane_end(self):
		pass

	def zerobridge_start(self, attr):
		points = [
			( float(attr["b_start_x"]), float(attr["b_start_y"]) ),
			( float(attr["b_ltop_x"]), float(attr["b_ltop_y"]) ),
			( float(attr["b_rtop_x"]), float(attr["b_rtop_y"]) ),
			( float(attr["b_end_x"]), float(attr["b_end_y"]) ),
		]
		try:
			self.topo.zerobridge.append(points)
		except AttributeError:
			self.topo.zerobridge = [ points ]
	def zerobridge_end(self):
		pass

	def feature_start(self, attr):
		points = [
			( float(attr["f_start_x"]), float(attr["f_start_y"]) ),
			( float(attr["f_ltop_x"]), float(attr["f_ltop_y"]) ),
			( float(attr["f_rtop_x"]), float(attr["f_rtop_y"]) ),
			( float(attr["f_end_x"]), float(attr["f_end_y"]) ),
			( float(attr["f_start_x"]), float(attr["f_start_y"]) ),
		]
		try:
			self.topo.feature.append(points)
		except AttributeError:
			self.topo.feature = [ points ]
	def feature_end(self):
		pass

	def entry_start(self, attr):
		loc = Location()
		loc.sequence = int(attr["position"])
		loc.type = attr["residue"]
		loc.x = float(attr["x"])
		loc.y = float(attr["y"])
		loc.symbol = attr["symbol"]
		loc.foreground = attr["foreground"]
		loc.background = attr["background"]
		loc.border = attr["border"]
		self.topo.locations.append(loc)
	def entry_end(self):
		pass

def str2rgb(s):
	return map(float, s.split())

def rgb2str(rgb):
	return "%g %g %g" % tuple(rgb)

